package edu.uoc.tfg.learningpoint.business.useradministration;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import edu.uoc.tfg.learningpoint.exception.LearningPointException;
import edu.uoc.tfg.learningpoint.exception.ValidationError;
import edu.uoc.tfg.learningpoint.integration.IUserJPA;

/**
 * Ticket
 * @author maip
 */
public class Ticket implements ITicket{

	
	/**
	 * Serial ID
	 */
	private static final long serialVersionUID = 8356380793488762500L;
	
	private String nif;
	
	private String email;
	
	private String password;
	
	private String name;
	
	private String surname;
	
	private String phone;
	
	private boolean isValid;
	
	private Date expirationDate;
	
	private boolean seller;

	
	Ticket( final IUserJPA userJPA ) {
		
		super();
		this.email = userJPA.getEmail();
		this.password = userJPA.getPassword();
		this.name = userJPA.getName();
		this.surname = userJPA.getSurname();
		this.phone = userJPA.getPhone();
		this.isValid = true;
		this.expirationDate = newExpirationDate();
		this.nif = userJPA.getNif();
		this.seller = userJPA.isSeller();
	}

	
	Ticket(String email, String password, boolean isValid, Date expirationDate) {
		super();
		this.email = email;
		this.password = password;
		this.isValid = isValid;
		this.expirationDate = expirationDate;
		this.nif = "";
		this.name = "";
		this.surname = "";
		this.phone= "";
		this.seller = false;
	}




	private Date newExpirationDate(){
		
		Calendar calendar = new GregorianCalendar();
		calendar.add(Calendar.MINUTE, 30);
		
		return calendar.getTime();
		
	}
	
	/**
	 * crea un ticket expirado para no trabajar con referencias nulas de tickets
	 * @param email
	 * @param password
	 * @return
	 */
	public static Ticket getInvalidTicket (final String email, final String password) {
		
		return new Ticket(email, password, false, new Date() );
		
	}

	@Override
	public String getEmail() {
		return email;
	}

	void setEmail(String email) {
		this.email = email;
	}

	@Override
	public String getPassword() {
		return password;
	}

	void setPassword(String password) {
		this.password = password;
	}
	
	@Override
	public String getName() {
		return name;
	}

	void setName(String name) {
		this.name = name;
	}
	
	@Override
	public String getSurname() {
		return surname;
	}

	void setSurname(String surname) {
		this.surname = surname;
	}
	
	@Override
	public String getPhone() {
		return phone;
	}

	void setPhone(String phone) {
		this.phone = phone;
	}

	@Override
	public boolean isValid() {
		return isValid;
	}

	void setValid(boolean isValid) {
		this.isValid = isValid;
	}

	@Override
	public Date getExpirationDate() {
		return expirationDate;
	}

	void setExpirationDate(Date expirationDate) {
		this.expirationDate = expirationDate;
	}
	
	@Override
	public String getId() {
		return Integer.toString( hashCode() );
	}
	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((email == null) ? 0 : email.hashCode());
		result = prime * result + ((password == null) ? 0 : password.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Ticket other = (Ticket) obj;
		if (email == null) {
			if (other.email != null)
				return false;
		} else if (!email.equals(other.email))
			return false;
		return true;
	}



	/**
	 * Indica si el cliente está autenticado en el sistema
	 * @throws LearningPointException
	 */
	@Override
	public void validateTicket() throws LearningPointException {
		
		if ( !this.isValid() ) {
			System.err.println( String.format (  "%s : %s ", this.getClass(), "Session de usuario no valida. Se debe logar en el sistema.") );
			throw new UserAdministrationException( new ValidationError("Session de usuario no valida. Se debe logar en el sistema."));
		}
		
	}


	@Override
	public String getNif() {
		return nif;
	}


	public void setNif(String nif) {
		this.nif = nif;
	}
	
	
	@Override
	public boolean isSeller(){
		return seller;
	}


	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("Ticket [");
		if (nif != null) {
			builder.append("nif=");
			builder.append(nif);
			builder.append(", ");
		}
		if (email != null) {
			builder.append("email=");
			builder.append(email);
			builder.append(", ");
		}
		if (password != null) {
			builder.append("password=");
			builder.append(password);
			builder.append(", ");
		}
		builder.append("isValid=");
		builder.append(isValid);
		builder.append(", ");
		if (expirationDate != null) {
			builder.append("expirationDate=");
			builder.append(expirationDate);
			builder.append(", ");
		}
		builder.append("seller=");
		builder.append(seller);
		builder.append("]");
		return builder.toString();
	}

	
	
	
	
}
