package edu.uoc.tfg.learningpoint.business.useradministration;

import java.io.Serializable;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.ParameterExpression;
import javax.persistence.criteria.Root;

import edu.uoc.tfg.learningpoint.exception.LearningPointException;
import edu.uoc.tfg.learningpoint.exception.ValidationError;
import edu.uoc.tfg.learningpoint.integration.SellerJPA;
import edu.uoc.tfg.learningpoint.integration.IUserJPA;
import edu.uoc.tfg.learningpoint.integration.StudentJPA;
import edu.uoc.tfg.learningpoint.integration.UserJPA;

/**
 * UserFacadeBean
 * @author maip
 */
@Stateless
public class UserFacadeBean implements UserFacadeLocal, UserFacadeRemote, Serializable{

	
	/**
	 * SerialID
	 */
	private static final long serialVersionUID = 4975767491245847174L;

	@PersistenceContext
	private EntityManager entityManager;
	
	private TicketStore ticketStore = TicketStore.getTicketStore();
	

	@Override
	public ITicket login( final String email, final String password ) throws LearningPointException {
		
		//http://docs.oracle.com/javaee/6/tutorial/doc/gjrij.html
		//http://www.objectdb.com/java/jpa/query/criteria
		
	    Ticket ticket = Ticket.getInvalidTicket(email, password);
		
		final List<UserJPA> results = getUserJPAByEmail(email, password);
		
		System.out.println(results);
		
		if ( results == null || results.size() == 0 ){
			System.err.println( String.format (  "%s : %s ", this.getClass(), "Usuario incorrecto. Introduzca los datos de nuevo.") );
			throw new UserAdministrationException( new ValidationError("Usuario incorrecto. Introduzca los datos de nuevo."));
		} ;
		
		ticket = new Ticket( results.get(0) );//crea un ticket valido
		ticketStore.addTicket(ticket);
		
		return ticket;
	}

	/**
	 * getUserJPAByEmail
	 * @param email
	 * @param password
	 * @return List<UserJPA>
	 */
	private List<UserJPA> getUserJPAByEmail ( final String email, final String password  ) {
		
		final CriteriaBuilder cb = entityManager.getCriteriaBuilder();
		
		final CriteriaQuery<UserJPA> cq = cb.createQuery(UserJPA.class);
		
		final Root<UserJPA> userJPA = cq.from(UserJPA.class);

		final ParameterExpression<String> emailPE = cb.parameter(String.class);
		
		final ParameterExpression<String> passwordPE = cb.parameter(String.class);
		
		if ( password != null ) {
			cq.where(cb.and( cb.equal( userJPA.get("email"), emailPE ), cb.equal( userJPA.get("password"), passwordPE ) )  );	
		} else {
			cq.where( cb.equal( userJPA.get("email"), emailPE )  );	
		}
		
		final TypedQuery<UserJPA> query = entityManager.createQuery(cq);
		
		query.setParameter(emailPE, email);
		
		if ( password != null ) {
			query.setParameter(passwordPE, password);
		}
		
		final List<UserJPA> results = query.getResultList();
		
		return results;
	}
	
	
	@Override
	public void logout( final ITicket ticket ) {
		
		ticketStore.removeTicket( ticket );
	}
	 
	@Override
	public void registerSeller ( final ITicket ticket, final SellerJPA sellerJPA ) throws LearningPointException {
		
		sellerJPA.validateCaseInsert();
		
		if ( entityManager.find(SellerJPA.class, sellerJPA.getNif()) != null ){
			System.err.println( String.format (  "%s : %s ", this.getClass(), "No ha sido posible persistir al vendedor. Ya existe otro vendedor con el mismo nif."));
			throw new UserAdministrationException( new ValidationError("No ha sido posible persistir al vendedor. Ya existe otro vendedor con el mismo nif."));
		}
		
		final List<UserJPA> results = getUserJPAByEmail( sellerJPA.getEmail(), null);
		
		if ( results !=null && results.size() > 0  ) {
			System.err.println( String.format (  "%s : %s ", this.getClass(), "No ha sido posible persistir al vendedor. Ya existe otro usuario con el mismo email."));
			throw new UserAdministrationException( new ValidationError("No ha sido posible persistir al vendedor. Ya existe otro usuario con el mismo email."));
		}
		
		entityManager.persist( sellerJPA );
		
	}
	
	@Override
	public void updatePersonalData ( final ITicket ticket, final IUserJPA userJPA ) throws LearningPointException {
		
		ticketStore.validateTicket(ticket);

		userJPA.validateCaseUpdate();
		
		if (!ticket.isSeller()) {
			entityManager.merge( new StudentJPA(userJPA) );
		} else {			
			entityManager.merge( new SellerJPA(userJPA) );
		}
		
	}		
	
	@Override
	public void registerStudent ( final ITicket ticket, final StudentJPA studentJPA ) throws LearningPointException {
		
		studentJPA.validateCaseInsert();
		
		if ( entityManager.find(StudentJPA.class, studentJPA.getNif()) != null ){
			System.err.println( String.format (  "%s : %s ", this.getClass(), "No ha sido posible persistir al estudiante. Ya existe otro estudiante con el mismo nif."));
			throw new UserAdministrationException( new ValidationError("No ha sido posible persistir al estudiante. Ya existe otro estudiante con el mismo nif."));
		}
		
		final List<UserJPA> results = getUserJPAByEmail( studentJPA.getEmail(), null);
		
		if ( results !=null && results.size() > 0  ) {
			System.err.println( String.format (  "%s : %s ", this.getClass(), "No ha sido posible persistir al pasajero. Ya existe otro usuario con el mismo email."));
			throw new UserAdministrationException( new ValidationError("No ha sido posible persistir al pasajero. Ya existe otro usuario con el mismo email."));
		}
		
		entityManager.persist( studentJPA );
		
	}
	
	@Override
	public UserJPA getUserJPA ( final ITicket ticket ) throws LearningPointException{
			
		ticketStore.validateTicket(ticket);
		
		return entityManager.find(UserJPA.class, ticket.getNif());
		
	}

	
	
}
