package edu.uoc.tfg.learningpoint.integration;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import edu.uoc.tfg.learningpoint.business.courseadministration.CourseAdministrationException;
import edu.uoc.tfg.learningpoint.exception.LearningPointException;
import edu.uoc.tfg.learningpoint.exception.ValidationError;

/**
 * JPA Class CourseJPA
 */
@Entity
@Table(name="learningpoint.course")

public class CourseJPA implements Serializable, IValidable{

	private static final long serialVersionUID = 1L;
	
	@Id
	@Column(name="id",columnDefinition = "serial")
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Integer id;
	
	private String name;
	private String description;
	private String area;
	private Date startDate;
	private Date endDate;
	private String url;
	private String place;
	private Integer seats;
	private Double price;
	
	@ManyToOne
	@JoinColumn(	name = "nif", 
					referencedColumnName = "nif")
	private SellerJPA sellerJPA;	
	
	
	@OneToMany (fetch = FetchType.LAZY, mappedBy = "courseJPA", cascade ={CascadeType.ALL })
	private Set<CourseStudentJPA> courseStudents;
	
	

	/**
	 * 
	 * Class constructor methods
	 *
	 */
	public CourseJPA() {
		super();
	}	


	
	public void update ( final CourseJPA courseJPA ) {
		
		update(
		courseJPA.getName(), courseJPA.getDescription(), courseJPA.getArea(), courseJPA.getStartDate(),
		courseJPA.getEndDate(), courseJPA.getUrl(), courseJPA.getPlace(), courseJPA.getSeats(), courseJPA.getPrice()
		);
	}
	
	public void update ( String name, String description, String area, Date startDate, Date endDate,
			String url, String place, Integer seats, Double price) {
		
		this.name = name;
		this.description = description;
		this.area = area;
		this.startDate = startDate;
		this.endDate = endDate;
		this.url = url;
		this.place = place;
		this.seats = seats;
		this.price = price;
		
	}



	@Override
	public void validateCaseInsert() throws LearningPointException {
		
		final List<ValidationError> errors = new ArrayList<ValidationError>();	
		
		if (name == null || name.equals("")) {
			errors.add( new ValidationError("name") );
		}
		if (description == null || description.equals("")) {
			errors.add( new ValidationError("description") );
		}
		if (area == null || area.equals("")) {
			errors.add( new ValidationError("area") );
		}
		if (startDate == null|| startDate.equals("")) {
			errors.add( new ValidationError("startDate") );
		}
		if (endDate == null|| endDate.equals("")) {
			errors.add( new ValidationError("endDate") );
		}
		if (url == null|| url.equals("")) {
			errors.add( new ValidationError("url") );
		}
		if (place == null|| place.equals("")) {
			errors.add( new ValidationError("place") );
		}
		if (seats == null|| seats.equals("")) {
			errors.add( new ValidationError("seats") );
		}
		if (price == null|| price.equals("")) {
			errors.add( new ValidationError("price") );
		}
		
		if ( errors.size() > 0 ) {
			System.err.println( String.format (  "%s : %s ", this.getClass(), errors));
			throw new CourseAdministrationException(errors);
		}	
	}
	
	@Override
	public void validateCaseUpdate() throws LearningPointException {		
		
		final List<ValidationError> errors = new ArrayList<ValidationError>();	
		
		if (id == null ) {
			errors.add( new ValidationError("id") );
		}
		if (name == null || name.equals("")) {
			errors.add( new ValidationError("name") );
		}
		if (description == null || description.equals("")) {
			errors.add( new ValidationError("description") );
		}
		if (area == null || area.equals("")) {
			errors.add( new ValidationError("area") );
		}
		if (startDate == null|| startDate.equals("")) {
			errors.add( new ValidationError("startDate") );
		}
		if (endDate == null|| endDate.equals("")) {
			errors.add( new ValidationError("endDate") );
		}
		if (url == null|| url.equals("")) {
			errors.add( new ValidationError("url") );
		}
		if (place == null|| place.equals("")) {
			errors.add( new ValidationError("place") );
		}
		if (seats == null|| seats.equals("")) {
			errors.add( new ValidationError("seats") );
		}		
		if (price == null|| price.equals("")) {
			errors.add( new ValidationError("price") );
		}
		
		if ( errors.size() > 0 ) {
			System.err.println( String.format (  "%s : %s ", this.getClass(), errors));
			throw new CourseAdministrationException(errors);
		}		
	}


	/**
	 * Get students from course
	 * @return
	 */
	public Set<StudentJPA> getStudents() {
		
		final Set<StudentJPA> studentsJPA = new HashSet<StudentJPA>();
		
		if ( courseStudents != null ) {
			
			Iterator<CourseStudentJPA> iterator = courseStudents.iterator();
			while ( iterator.hasNext() ) {
				CourseStudentJPA courseStudentJPA = iterator.next();
				studentsJPA.add( courseStudentJPA.getStudentJPA() );
			}
		}
		
		return studentsJPA;
	}
	
	

	public Integer getId() {
		return id;
	}



	public void setId(Integer id) {
		this.id = id;
	}

	
	public String getName() {
		return name;
	}



	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}



	public void setDescription(String description) {
		this.description = description;
	}



	public String getArea() {
		return area;
	}



	public void setArea(String area) {
		this.area = area;
	}



	public Date getStartDate() {
		return startDate;
	}



	public void setStartDate(Date startDate) {
		this.startDate = startDate;
	}



	public Date getEndDate() {
		return endDate;
	}



	public void setEndDate(Date endDate) {
		this.endDate = endDate;
	}


	public String getUrl() {
		return url;
	}


	public void setUrl(String url) {
		this.url = url;
	}



	public String getPlace() {
		return place;
	}


	public void setPlace(String place) {
		this.place = place;
	}



	public Integer getSeats() {
		return seats;
	}


	public void setSeats(Integer seats) {
		this.seats = seats;
	}



	public Double getPrice() {
		return price;
	}



	public void setPrice(Double price) {
		this.price = price;
	}



	public SellerJPA getSellerJPA() {
		return sellerJPA;
	}



	public void setSellerJPA(SellerJPA sellerJPA) {
		this.sellerJPA = sellerJPA;
	}



	public Set<CourseStudentJPA> getCourseStudents() {
		return courseStudents;
	}



	public void setCourseStudents(Set<CourseStudentJPA> courseStudents) {
		this.courseStudents = courseStudents;
	}



	
}
