package edu.uoc.tfg.learningpoint.integration;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import edu.uoc.tfg.learningpoint.business.useradministration.UserAdministrationException;
import edu.uoc.tfg.learningpoint.exception.LearningPointException;
import edu.uoc.tfg.learningpoint.exception.ValidationError;

/**
 * 
 * JPA Class SellerJPA
 *
 */
@Entity
@Table(name="learningpoint.seller")

public class SellerJPA implements IUserJPA {
	
	/**
	 * discriminator
	 */
	public static final String USER_TYPE = "seller";
	
	private static final long serialVersionUID = 1L;
	
	@Id
	private String nif;
	private String name;
	private String surname;
	private String phone;
	private String password;
	private String email;
	@javax.persistence.Transient
	private String userType = USER_TYPE;
	
	
	@OneToMany (fetch = FetchType.LAZY, mappedBy = "sellerJPA", cascade = CascadeType.ALL)
	private Set<CourseJPA> courses;




	/**
	 * 
	 * Class constructor methods
	 *
	 */
	public SellerJPA() {
		super();
	}	
	
	/**
	 * constructor insert/update
	 * @param nif
	 * @param name
	 * @param surname
	 * @param phone
	 * @param password
	 * @param email
	 */
	public SellerJPA( String nif, String name, String surname, String phone, 
			String password, String email ) throws LearningPointException {
		
		this.nif = nif;
		this.name = name;
		this.surname = surname;
		this.phone = phone;
		this.password = password;
		this.email = email;
		validate();
	}
	
	public SellerJPA ( IUserJPA userJPA ) throws LearningPointException {
		
		this (userJPA.getNif(), userJPA.getName(), userJPA.getSurname(), userJPA.getPhone(), userJPA.getPassword(), userJPA.getEmail() );
	}
	
	/**
	 * 
	 * Methods get/set the fields of database
	 * Id Primary Key field
	 */
	@Override
	public String getNif() {
		return nif;
	}
	@Override
	public void setNif(String nif) {
		this.nif = nif;		
	}
	
	@Override
	public String getName(){
		return name;
	}
	@Override
	public void setName(String name){
		this.name = name;
	}
	
	@Override
	public String getSurname(){
		return surname;
	}
	@Override
	public void setSurname(String surname){
		this.surname = surname;
	}
	
	@Override
	public String getPhone(){
		return phone;
	}
	@Override
	public void setPhone(String phone){
		this.phone = phone;
	}
	
	@Override
	public String getPassword(){
		return password;
	}
	@Override
	public void setPassword(String password){
		this.password = password;
	}
	
	@Override
	public String getEmail(){
		return email;
	}
	@Override
	public void setEmail(String email){
		this.email = email;
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("SellerJPA [");
		if (nif != null) {
			builder.append("nif=");
			builder.append(nif);
			builder.append(", ");
		}
		if (name != null) {
			builder.append("name=");
			builder.append(name);
			builder.append(", ");
		}
		if (surname != null) {
			builder.append("surname=");
			builder.append(surname);
			builder.append(", ");
		}
		if (phone != null) {
			builder.append("phone=");
			builder.append(phone);
			builder.append(", ");
		}
		if (password != null) {
			builder.append("password=");
			builder.append(password);
			builder.append(", ");
		}
		if (email != null) {
			builder.append("email=");
			builder.append(email);
		}
		builder.append("]");
		return builder.toString();
	}
	
	
	private void validate() throws LearningPointException {
		
		final List<ValidationError> errors = new ArrayList<ValidationError>();
		
		if (nif == null || nif.equals("")) {
			errors.add( new ValidationError("nif") );
		}
		if (name == null || name.equals("")) {
			errors.add( new ValidationError("name") );
		}
		if (surname == null|| surname.equals("")) {
			errors.add( new ValidationError("surname") );
		}
		if (phone == null|| phone.equals("")) {
			errors.add( new ValidationError("phone") );
		}
		if (password == null|| password.equals("")) {
			errors.add( new ValidationError("password") );
		}
		if (email == null|| email.equals("")) {
			errors.add( new ValidationError("email") );
		}
		
		if ( errors.size() > 0 ) {
			System.err.println( String.format ( "%s : %s ", this.getClass(), errors));
			throw new UserAdministrationException(errors);
		}
	}

	@Override
	public void validateCaseInsert() throws LearningPointException {
		validate();	
	}

	@Override
	public void validateCaseUpdate() throws LearningPointException {
		validate();
	}
	
	
	@Override
	public String getUserType() {
		return userType;
	}

	@Override
	public void setUserType(String userType) {
		this.userType = userType;
	}
	
	@Override
	public boolean isSeller(){
		return true;
	}

	public Set<CourseJPA> getCourses() {
		if (courses == null) {
			courses = new HashSet<CourseJPA>();
		}
		for (CourseJPA courseJPA : courses) {
			courseJPA.getId();//inicializa los cursos, evita la excepcion de lazyInitializationException. No evitamos poner la relacion como eager
		}
		return courses;
	}

	public void setCourses(Set<CourseJPA> courses) {
		this.courses = courses;
	}
	
	public void add (final CourseJPA courseJPA){
		this.getCourses().add(courseJPA);
		courseJPA.setSellerJPA(this);
	}
}


